/*
    Copyright (c) 2009 Charles Wautier <charles.wautier@gmail.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "newsitedialog.h"
#include "konquerortaskplugin.h"
#include "ui_newsitedialog.h"
#include "tmo.h"
#include "tasktreedialog.h"

#include "pimo.h"

#include <soprano/nao.h>
#include <soprano/rdf.h>
#include <Soprano/Node>
#include <Soprano/Model>
#include <Soprano/StatementIterator>
#include <Soprano/QueryResultIterator>
#include <Soprano/Error/Error>

#include "nepomukcontext.h"
#include <Nepomuk/ResourceManager>
#include <Nepomuk/Thing>
#include <Nepomuk/Resource>
#include <Nepomuk/Variant>

#include <KToolBarPopupAction>
#include <KAction>
#include <KActionCollection>
#include <KLocale>
#include <KIcon>
#include <KMessageBox>
#include <KPluginFactory>
#include <KPluginLoader>
#include <KTemporaryFile>
#include <KDialog>

#include <QMenu>
#include <QUrl>
#include <QLabel>
#include <QtDBus/QDBusMessage>
#include <QtDBus/QDBusConnection>

#include <kmessagebox.h>
#include <khtml_part.h>
#include <dom/html_document.h>
#include <dom/dom_string.h>

#define NFO_WEBSITE_TYPE "http://www.semanticdesktop.org/ontologies/2007/03/22/nfo#Website"

Nepomuk::KonquerorTaskPlugin::KonquerorTaskPlugin(QObject* parent, const QVariantList&)
        : KParts::Plugin(parent)
{
    // Configure the button
    taskAction = new KToolBarPopupAction(KIcon("view-calendar-tasks"), i18n("Action"), actionCollection());
    taskAction->setText(i18n("Associate the current page to a task"));

    actionCollection()->addAction("nepomuk_link_to_task", taskAction);

    connect(taskAction, SIGNAL(triggered(Qt::MouseButtons, Qt::KeyboardModifiers)),
            this, SLOT(slotLinkToCurrentTask(Qt::MouseButtons, Qt::KeyboardModifiers)));
    connect(taskAction->menu(), SIGNAL(aboutToShow()),
            this, SLOT(slotUpdateTaskList()));

    // Configure the connection to the context service
    m_context = new Nepomuk::ContextServiceInterface();
    m_currentContext = m_context->currentContext();
    connect(m_context, SIGNAL(currentContextChanged(Nepomuk::Resource)),
            this, SLOT(slotContextChanged(Nepomuk::Resource)));

    // Make the connection with Nepomuk server
    if (Nepomuk::ResourceManager::instance()->init()) {
        //FIXME : Errormessage in dialog box ?
        //KMessageBox::error( 0,
        //        i18n("<b>Unable to join nepomukserver</b><br>Task Plugin won't work") );
        //kError() << "Unable to initialize KMetaData";
        delete taskAction;
    }
}

Nepomuk::KonquerorTaskPlugin::~KonquerorTaskPlugin()
{
}

void Nepomuk::KonquerorTaskPlugin::slotContextChanged(Nepomuk::Resource res)
{
    m_currentContext = res;
}

void Nepomuk::KonquerorTaskPlugin::slotUpdateTaskList()
{

    // Clear the existing task list
    taskAction->menu()->clear();

    // Load the task list with the most recently used tasks

    //// Unordered version:
    //Soprano::StatementIterator it =
    //    Nepomuk::ResourceManager::instance()->mainModel()->listStatements(Soprano::Node(),
    //                                                                      Soprano::Node(Soprano::Vocabulary::RDF::type()),
    //                                                                      Soprano::Node(Nepomuk::Vocabulary::TMO::Task()));
    //for ( int i = 0 ; i < 9 && it.next() ; ++i )
    //{
    //    Nepomuk::Resource res((*it).subject().uri());
    //    QAction* taskEntry = new QAction( KIcon(res.genericIcon()), res.label(), this );
    //    taskEntry->setData( QVariant( res.resourceUri().toString() ) );
    //
    //    connect( taskEntry, SIGNAL( triggered() ),
    //             this, SLOT( slotLinkToTask() ) );
    //    taskAction->menu()->addAction( taskEntry );
    //}
    //taskAction->menu()->addSeparator();
    //taskAction->menu()->addAction( KIcon("nepomuk"), QString(i18n("Other ...")), this, SLOT(slotLinkToSelectedTask() ) );

    //// Ordered version :
    // Make the query
    QString query = QString("select distinct ?r where { "
                            "?r <%1> <%2> . "
                            "?r <%3> ?d . }"
                            "order by DESC(?d)")  // ORDER BY creation date
                    .arg(Soprano::Vocabulary::RDF::type().toString())
                    .arg(Nepomuk::Vocabulary::TMO::Task().toString())
                    .arg(Soprano::Vocabulary::NAO::created().toString());

    // Parse the result
    Soprano::QueryResultIterator it =
        Nepomuk::ResourceManager::instance()->mainModel()->executeQuery(query, Soprano::Query::QueryLanguageSparql);

    // Add the tasks into the pop-up menu
    for (int i = 0 ; i < 9 && it.next() ; ++i) {
        Nepomuk::Resource res(it.binding("r").uri());
        QAction* taskEntry = new QAction(KIcon(res.genericIcon()), res.genericLabel(), this);
        taskEntry->setData(QVariant(res.resourceUri()));

        connect(taskEntry, SIGNAL(triggered()),
                this, SLOT(slotLinkToTask()));
        taskAction->menu()->addAction(taskEntry);
    }

    //
    taskAction->menu()->addSeparator();
    taskAction->menu()->addAction(KIcon("nepomuk"), i18n("Other ..."), this, SLOT(slotLinkToSelectedTask()));
}

void Nepomuk::KonquerorTaskPlugin::slotLinkToCurrentTask(Qt::MouseButtons, Qt::KeyboardModifiers)
{
    if (m_currentContext.isValid()) {
        doLinkToTask(m_currentContext);
    } else {
        slotLinkToSelectedTask();
    }
}

void Nepomuk::KonquerorTaskPlugin::slotLinkToTask()
{
    Nepomuk::Resource res(static_cast<QAction*>(sender())->data().toUrl());//value<KUrl>());
    doLinkToTask(res);
}

void Nepomuk::KonquerorTaskPlugin::slotLinkToSelectedTask()
{
    // Parameters are just the title and the parent of the dialog window
    Nepomuk::Resource res = TaskTreeDialog::selectOneTask(taskAction->menu());
    if (res.isValid()) {
        doLinkToTask(res);
    }
}

void Nepomuk::KonquerorTaskPlugin::doLinkToTask(Nepomuk::Resource &tsk) const
{
    if (parent() && parent()->inherits("KHTMLPart")) {

        KHTMLPart* htmlPart = static_cast<KHTMLPart*>(parent());

        // Get the resource corresponding to the url if it exists.
        // If it does nt exists yet, it create the resource, but don't add it in Nepomuk database.
        Nepomuk::Resource nfoResource(htmlPart->url(), QUrl::fromEncoded(NFO_WEBSITE_TYPE, QUrl::StrictMode));

        // Configure the dialog
        NewSiteDialog dlg(taskAction->menu());
        dlg.setCaption(i18n("Associate Page to Task"));
        dlg.m_url->setText(htmlPart->url().url());
        dlg.m_url->setText(htmlPart->url().url());
        dlg.m_title->setText(i18nc("@title:window", "Link page"));
        dlg.m_title->setComment(i18n("to the task ") + QString("<i>%1</i>").arg(tsk.label()));

        bool exists = nfoResource.exists();
        if (exists) {
            dlg.m_nameLabel->setText(nfoResource.genericLabel());
        } else {
            dlg.m_nameLabel->setText(htmlPart->htmlDocument().title().string().trimmed());
            dlg.m_nameLabel->toggleEditor(true);
        }

        // If Accepted, do link the page to the task (and create the page corresponding resource if needed)
        // Else, we have not create the resource corresponding to the page, since a new Nepomuk::Resource X
        // is exported in the database only if we modify it, by X.addProperty( ... ) for example.
        if (dlg.exec() == QDialog::Accepted) {
            Nepomuk::Thing webPageThing = nfoResource.pimoThing();
            nfoResource.addProperty(Nepomuk::Vocabulary::PIMO::isRelated(), tsk);
            nfoResource.setLabel(dlg.m_nameLabel->text());
            nfoResource.addSymbol("text-html");
        }
    }
}

K_PLUGIN_FACTORY(KonquerorTaskPluginFactory, registerPlugin<Nepomuk::KonquerorTaskPlugin>();)
K_EXPORT_PLUGIN(KonquerorTaskPluginFactory("khtml_nepomuk_task_linker"))

#include "konquerortaskplugin.moc"
