/*
    Copyright (c) 2009 Charles Wautier <charles.wautier@gmail.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _KONQUEROR_TASK_PLUGIN_H_
#define _KONQUEROR_TASK_PLUGIN_H_

#include <kparts/plugin.h>
#include <Nepomuk/Resource>
#include "nepomukcontext.h"

class KToolBarPopupAction;

namespace Nepomuk
{

/**
 * A button inserted in Konqueror's ToolBar to link the current webpage to a task
 */
class KonquerorTaskPlugin : public KParts::Plugin
{
    Q_OBJECT

public:

    KonquerorTaskPlugin(QObject* parent, const QVariantList& args = QVariantList());
    ~KonquerorTaskPlugin();

private Q_SLOTS:

    /**
     * The 'context' is a particular resource, managed by a NepomukService,
     * so we can access and set it from any program.
     * Changes m_currentContext.
     */
    void slotContextChanged(Nepomuk::Resource);
    /**
     * Update the list of the 9 tasks the most recently used before the user
     * click on the pop-up menu.
     */
    void slotUpdateTaskList();

    /**
     * One slot per type of link button. They get the appropriate task and
     * give it to doLinkTask which will make the effective job.
     */
    void slotLinkToCurrentTask(Qt::MouseButtons, Qt::KeyboardModifiers);
    void slotLinkToTask();
    void slotLinkToSelectedTask();

private:

    Nepomuk::ContextServiceInterface* m_context;
    Nepomuk::Resource m_currentContext;
    KToolBarPopupAction* taskAction;

    /**
     * Link the page to the task.
     */
    void doLinkToTask(Nepomuk::Resource &tsk) const;

};
}

#endif
