/*
   Copyright (C) 2008-2009 by Sebastian Trueg <trueg at kde.org>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "resourcecompletion.h"

#include <Nepomuk/ResourceManager>
#include <Nepomuk/Resource>
#include <Nepomuk/Types/Class>

#include <Soprano/QueryResultIterator>
#include <Soprano/Model>
#include <Soprano/BindingSet>
#include <Soprano/Vocabulary/RDFS>
#include <Soprano/Vocabulary/RDF>
#include <Soprano/Index/IndexFilterModel>

#include <kdebug.h>
#include <kicon.h>


class Nepomuk::ResourceCompletion::Private
{
public:
    Private()
        : type( Soprano::Vocabulary::RDFS::Resource() ) {
    }

    Types::Class type;
};


Nepomuk::ResourceCompletion::ResourceCompletion( QObject* parent )
    : KCompleter(),
      d( new Private() )
{
    setParent( parent );
}


Nepomuk::ResourceCompletion::~ResourceCompletion()
{
    delete d;
}


Nepomuk::Types::Class Nepomuk::ResourceCompletion::type() const
{
    return d->type;
}


void Nepomuk::ResourceCompletion::makeCompletion( const QString& string )
{
    kDebug() << string;

    if ( string.length() > 2 ) {
        QList<Types::Class> typeOffspring = d->type.allSubClasses();
        QStringList typeQueries;
        foreach( const Types::Class& type, ( typeOffspring << d->type ) ) {
            typeQueries << ( Soprano::Index::IndexFilterModel::encodeUriForLuceneQuery( Soprano::Vocabulary::RDF::type() ) + ':' +
                             Soprano::Index::IndexFilterModel::encodeUriForLuceneQuery( type.uri() ) );
        }
        QString query = QString( "%1* AND (%2)" ).arg( string ).arg( typeQueries.join( QLatin1String( " OR " ) ) );
        kDebug() << query;
        Soprano::QueryResultIterator it
            = ResourceManager::instance()->mainModel()->executeQuery( query,
                                                                      Soprano::Query::QueryLanguageUser,
                                                                      "lucene" );
        int cnt = 0;
        while ( it.next() ) {
            Resource res( it.binding( 0 ).uri() );
            double score = it[1].literal().toDouble();
            kDebug() << "Match for input" << string << res.uri();
            addCompletion( KCompletionItem( res.genericLabel(),
                                            QString( "%1 (%2)" ).arg( res.genericLabel() ).arg( Types::Class( res.type() ).label() ),
                                            res.genericDescription(),
                                            KIcon( res.genericIcon() ),
                                            score,
                                            res.resourceUri() ) );

            if ( ++cnt >= 10 ) {
                kDebug() << "Stopping at" << cnt << "results";
                return;
            }
        }
    }
}


void Nepomuk::ResourceCompletion::setType( const Nepomuk::Types::Class& type )
{
    d->type = type;
}


void Nepomuk::ResourceCompletion::itemActivated( const KCompletionItem& item )
{
    emit resourceActivated( item.userData().toUrl() );
}


void Nepomuk::ResourceCompletion::itemHighlighted( const KCompletionItem& item )
{
    emit resourceHighlighted( item.userData().toUrl() );
}

#include "resourcecompletion.moc"
