/*
   Copyright (C) 2009 by Dupuy Mathieu <mdupuy at mandriva.com>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include <KAboutData>
#include <KApplication>
#include <KCmdLineArgs>
#include <KDialog>
#include <KUrl>
#include <KMessageBox>

#include <Nepomuk/Variant>
#include <Nepomuk/ResourceManager>

#include "pimo.h"
#include "nfo.h"
#include "tasklinker.h"

#include <kdebug.h>

int main( int argc, char *argv[] )
{
  KAboutData aboutData( "tasklinker",
                        "tasklinker",
                        ki18n("Tasklinker"),
                        "0.1",
                        ki18n("A tool for associating resources to tasks"),
                        KAboutData::License_GPL,
                        ki18n("(c) 2009, Mathieu Dupuy"),
                        KLocalizedString(),
                        "http://nepomuk.kde.org" );
  aboutData.addAuthor(ki18n("Mathieu Dupuy"),ki18n("Author"), "mdupuy@mandriva.org");
  aboutData.setProgramIconName( "nepomuk" );

  KCmdLineArgs::init( argc, argv, &aboutData );

  KCmdLineOptions options;
  options.add("+uri", ki18n("The URI of the resource to associate"));
  KCmdLineArgs::addCmdLineOptions( options );

  KCmdLineArgs *args = KCmdLineArgs::parsedArgs();

  KApplication app;
  Nepomuk::ResourceManager::instance()->init();
  KUrl url = args->url(0);

  if ( url.protocol() == "nepomuktree" ) {
    KMessageBox::error( NULL, i18n( "Tasks cannot be associated to each other." ) );
    app.exit(1);
  }

  Nepomuk::Resource     resource;
  if ( url.protocol() == "file" )
    resource = Nepomuk::Resource( url, Nepomuk::Vocabulary::NFO::FileDataObject() );
  else
    resource =Nepomuk::Resource( url );

  kDebug() << "Resource exists ?" << resource.exists() << " is valid " << resource.isValid();
    
  TaskLinker* tasklinker = new TaskLinker;
  tasklinker->setTitleAndIcon( resource );

  if ( tasklinker->exec() == KDialog::Accepted)
    {
      Nepomuk::Resource Task = tasklinker->getSelectedResource();

      QString label;
      if ( !resource.exists() ) {
        label = url.fileName();
        resource.setLabel( label );
      }
      else 
        label = resource.genericLabel();

      resource.addProperty( Nepomuk::Vocabulary::PIMO::isRelated(), Task);
      
      if (resource.hasProperty( Nepomuk::Vocabulary::PIMO::isRelated() ) )
        KMessageBox::information(tasklinker, i18n("'%1' has been associated to '%2'", label, Task.genericLabel() ), i18n( "Association created successfully." ) ); 
      else 
        KMessageBox::error(tasklinker, i18n("'%1' could not be associated to '%2'", label, Task.genericLabel() ) );
    }
  return 0;
}
