/*
   Copyright (C) 2009 by Dupuy Mathieu <mdupuy at mandriva.com>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include <Nepomuk/Resource>
#include <Nepomuk/ResourceManager>
#include <Nepomuk/Variant>

#include <Soprano/Model>
#include <Soprano/StatementIterator>
#include <Soprano/Vocabulary/RDF>

#include <QVBoxLayout>
#include <KTitleWidget>
#include <KLocalizedString>
#include <kresourcetreeview.h>

#include "tasklinker.h"
#include "pimo.h"
#include "tmo.h"
#include <kdebug.h>


TaskLinker::TaskLinker(QWidget* parent) : KDialog(parent)
{
  QWidget* w = new QWidget(this);
  QVBoxLayout* verticalLayout = new QVBoxLayout(w);

  m_labeltitle = new KTitleWidget(w);
  m_taskview = new KResourceTreeView(w);

  verticalLayout->addWidget(m_labeltitle);
  verticalLayout->addWidget(m_taskview);
  w->setLayout(verticalLayout);
  setMainWidget(w);

  m_taskview->setDirOnlyMode( true );
  for (int nb = m_taskview->model()->columnCount(); nb > 0; --nb)
    m_taskview->setColumnHidden( nb, true );

  enableButtonOk( false );
  connect (m_taskview, SIGNAL( currentUriChanged( const KUrl& ) ),
           this, SLOT( enableOkButton( ) ));
}

void TaskLinker::enableOkButton()
{
  enableButtonOk( true );
  disconnect (m_taskview, SIGNAL( currentUriChanged( const KUrl& ) ),
           this, SLOT( enableOkButton( ) ));
}

void TaskLinker::setTitleAndIcon(const Nepomuk::Resource& resource)
{
  QString label;
  KIcon icon;
  if ( !resource.exists() ) {
    label = KUrl( resource.resourceUri()).fileName();
    icon = KIcon( "nepomuk" );
  }
  else {
    label = resource.genericLabel();
    icon = KIcon(resource.genericIcon());
  }

  m_labeltitle->setText( i18n( "Associate file <i>%1</i> to", label) );
  m_labeltitle->setPixmap( icon.pixmap( 32, 32 ) );
}


QList<Nepomuk::Resource> TaskLinker::getTasks()
{
  Soprano::StatementIterator it = 
    Nepomuk::ResourceManager::instance()->mainModel()->listStatements(QUrl(""),
                                                                      Soprano::Vocabulary::RDF::type(),
                                                                      Nepomuk::Vocabulary::TMO::Task());
  QList<Nepomuk::Resource> resources;
  while (it.next()) {
    resources << (*it).subject().uri();
  }
  return resources;
}

Nepomuk::Resource       TaskLinker::getSelectedResource()
{
  return m_taskview->currentUri();
}

#include "tasklinker.moc"
