/*
   Copyright (C) 2009 by Dupuy Mathieu <mdupuy at mandriva.com>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include <QMap>
#include <QListView>
#include <QSortFilterProxyModel>
#include <KUrl>
#include <kdirnotify.h>

#include <Soprano/QueryResultIterator>
#include <Soprano/QueryResultIterator>
#include <Soprano/StatementIterator>
#include <Soprano/Model>
#include <Soprano/Vocabulary/RDF>
#include <Soprano/NodeIterator>
#include <Nepomuk/Resource>
#include <Nepomuk/ResourceManager>
//#include <Nepomuk/Types/Class>
#include <nepomuk/class.h>

#include <kdebug.h>
#include "simpleresourcemodel.h"
#include "pimo.h"
#include "nfo.h"
#include "tmo.h"
#include "nco.h"
#include "controller.h"

static Nepomuk::Types::Class equivalence[] = {
  Nepomuk::Vocabulary::NCO::Contact(),
  Nepomuk::Vocabulary::NFO::FileDataObject(),
  KUrl("http://www.semanticdesktop.org/ontologies/2007/03/22/nmo#Email"),
  Nepomuk::Vocabulary::NFO::Website() };

Controller::Controller(QObject* parent) :
  QObject(parent)
{
  connect( Nepomuk::ResourceManager::instance()->mainModel(), SIGNAL( statementAdded( const Soprano::Statement& ) ),
           this, SLOT( statementAdded( const Soprano::Statement& )) );

  connect( Nepomuk::ResourceManager::instance()->mainModel(), SIGNAL( statementRemoved( const Soprano::Statement&  )),
           this, SLOT( statementRemoved( const Soprano::Statement& )) );

  // connect( Nepomuk::ResourceManager::instance()->mainModel(), SIGNAL( statementsAdded( ) ),
  //          this, SLOT( dataInBDModified() )); //statementsAdded(  )) );

  // connect( Nepomuk::ResourceManager::instance()->mainModel(), SIGNAL( statementsRemoved(  )),
  //          this, SLOT( statementsRemoved(  )) );
}

void Controller::addModel(QListView* listview, ModelType type) {
  Nepomuk::SimpleResourceModel* resourceModel = new Nepomuk::SimpleResourceModel(this);
  QSortFilterProxyModel* proxyModel = new QSortFilterProxyModel(listview);
  proxyModel->setSourceModel(resourceModel);
  listview->setModel(proxyModel);
  m_resourceModelMap[type] = resourceModel;
}

void Controller::slot_currentChanged(const KUrl& Task)
{
  Nepomuk::Resource resource(Task);

  QMap<ModelType, Nepomuk::SimpleResourceModel*>::const_iterator it = m_resourceModelMap.constBegin();
  const QMap<ModelType, Nepomuk::SimpleResourceModel*>::const_iterator it_end = m_resourceModelMap.constEnd();
  while (it != it_end) {
    Nepomuk::Types::Class type = equivalence[it.key()];

    Soprano::QueryResultIterator qit =
      Nepomuk::ResourceManager::instance()->mainModel()->executeQuery( QString("select distinct ?r where { ?r a %1 . ?r %2 %3 }")
                                                                       .arg(Soprano::Node::resourceToN3(type))
                                                                       .arg(Soprano::Node::resourceToN3(Nepomuk::Vocabulary::PIMO::isRelated()))
                                                                       .arg(Soprano::Node::resourceToN3(resource.resourceUri())),
                                                                      Soprano::Query::QueryLanguageSparql);

    QList<Nepomuk::Resource> resources;
    while (qit.next()) {
      resources << qit["r"].uri();
    }
    (m_resourceModelMap[it.key()])->setResources(resources);
    ++it;
  }
}

QList<Nepomuk::Resource> Controller::getResourcesOfTypeWithExclusion(const Nepomuk::Types::Class& type, const KUrl& resource)
{
  kDebug() << QString("select distinct ?r where { ?r a %1 optional { ?r %2 %3 } filter ( !bound (%3) ) }") .
    arg(Soprano::Node::resourceToN3(type.uri()))
    .arg(Soprano::Node::resourceToN3(Nepomuk::Vocabulary::PIMO::isRelated()))
    .arg(Soprano::Node::resourceToN3(resource));
  
  Soprano::QueryResultIterator qit = Nepomuk::ResourceManager::instance()->mainModel()->executeQuery(
                                                                                                     QString("select distinct ?r where { ?r a %1 optional { ?r %2 ?t } filter ( !bound (?t) || ?t!=%3 )}")
                                                                                                     .arg(Soprano::Node::resourceToN3(type.uri()))
                                                                                                     .arg(Soprano::Node::resourceToN3(Nepomuk::Vocabulary::PIMO::isRelated()))
                                                                                                     .arg(Soprano::Node::resourceToN3(resource)),
                                                                                                     Soprano::Query::QueryLanguageSparql);

  QList<Nepomuk::Resource> resources;
  while (qit.next()) {
    resources << qit["r"].uri();
    //kDebug() << qit["r"].uri();
  }
  return resources;
}

KUrl Controller::FromUriToNepomukTreeUrl(const KUrl& daughter)
{
  Soprano::Model* model = Nepomuk::ResourceManager::instance()->mainModel();

  if (model->containsAnyStatement(daughter,
                                  Nepomuk::Vocabulary::TMO::superTask(),
                                  Soprano::Node::createEmptyNode())) {

    Soprano::StatementIterator statIt = model->listStatements(daughter,
                                                              Nepomuk::Vocabulary::TMO::superTask(),
                                                              Soprano::Node::createEmptyNode());
    statIt.next();

    KUrl mother = statIt.iterateObjects().current().toString();
    KUrl motherArbo = Controller::FromUriToNepomukTreeUrl(mother);

    motherArbo.addPath( daughter.url().toAscii().toPercentEncoding("", "~", '~') );//concatene a resource URI in form nepomuk~3A~2Fresource
    //kDebug() << "final url" << motherArbo.url() << "current uri" << daughter.url();
    return motherArbo;
  }
  KUrl GrandMother;
  GrandMother.setProtocol("nepomuktree");
  GrandMother.adjustPath(KUrl::AddTrailingSlash);
  GrandMother.addPath( daughter.url().toAscii().toPercentEncoding("", "~", '~')  );
  return GrandMother;
}

void Controller::statementAdded(const Soprano::Statement& stat)
{
  if (stat.object().toString() == Nepomuk::Vocabulary::TMO::Task().toString()) {
    KUrl url = FromUriToNepomukTreeUrl(stat.subject().toString());
    kDebug() << "Task uri:" << stat.subject() << " | Task nepomktree url:" << url;
    org::kde::KDirNotify::emitFilesAdded(url.url());
  }
}

void Controller::statementRemoved(const Soprano::Statement& stat)
{
  if (stat.object().toString() == Nepomuk::Vocabulary::TMO::Task().toString()) {
    KUrl url = FromUriToNepomukTreeUrl(stat.subject().toString());
    kDebug() << "Task uri:" << stat.subject() << " | Task nepomktree url:" << url;
    org::kde::KDirNotify::emitFilesRemoved( QStringList( url.directory() )); //FIXME: to be tested
  }
}

// void    Controller::removeTaskRelation(const KUrl& resource)
// {
//    // Soprano::QueryResultIterator qit =
//    //    Nepomuk::ResourceManager::instance()->mainModel()->executeQuery( QString("select ?q ?r where { ?q %2 ?r }")
//    //                                                                     .arg(Soprano::Node::resourceToN3(Nepomuk::Vocabulary::PIMO::isRelated()))
//    //                                                                     .arg(Soprano::Node::resourceToN3(resource)),
//    //                                                                    Soprano::Query::QueryLanguageSparql );

//   Soprano::QueryResultIterator qit =
//     Nepomuk::ResourceManager::instance()->mainModel()->removeAllStatement(QUr(""),
//                                                                           Nepomuk::Vocabulary::PIMO::isRelated(),
//                                                                           resourceToN3( resource ));

// }
