/*
  Copyright (C) 2009 by Dupuy Mathieu <mdupuy at mandriva.com>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include <KProcess>
#include <KFileDialog>
#include <KMessageBox>
#include <KRun>
#include <KDebug>
#include <kdirnotify.h>
// for the about window
#include <kglobal.h>
#include <kcomponentdata.h>
#include <kaboutapplicationdialog.h>

#include <Nepomuk/Variant>
#include <Nepomuk/Resource>
#include <nepomuk/class.h>//<Nepomuk/Types/Class>
#include <Nepomuk/ResourceManager> // should not be there !
#include <Soprano/Model>          // this one neither

#include <QMenu>
#include <QInputDialog>
#include <QList>
#include <QToolButton>
#include <QItemSelectionModel>
#include <QtGui>

#include "simpleresourcemodel.h"
#include "newtaskdialog.h"
#include "newpersondialog.h"
#include "neworganizationdialog.h"

#include "task-import-export.h"
#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "simpleresourcelistview.h"
#include "pimo.h"
#include "controller.h"
#include "nfo.h"
#include "tmo.h"
#include "nco.h"

//TODO:fix redundance between local signal for synching with DB and soprano model signals

static const char* nmoMessageId = "http://www.semanticdesktop.org/ontologies/2007/03/22/nmo#messageId";

inline QList<Nepomuk::Resource> getListSelectedResources(QListView* view)
{
    QListIterator<QModelIndex> i(view->selectionModel()->selectedRows());
    QList<Nepomuk::Resource> resourceList;

    while (i.hasNext()) {
        resourceList << i.next().data(Nepomuk::ResourceModel::ResourceUriRole).toUrl();
    }
    return resourceList;
}

void MainWindow::setupButtons()
{//TODO: A finir
    //QListIterator<QAbstractButton*> it( m_ui->m_removeButtons->buttons() );
    //while ( it.hasNext() )
    // for ( QListIterator<QAbstractButton*> it( m_ui->m_removeButtons->buttons() ); it.hasNext() ;  )
    //   it.next()->setIcon( KIcon( "list-add" ) );

    m_ui->m_addPerson->setIcon(KIcon("list-add"));
    m_ui->m_addMail->setIcon(KIcon("list-add"));
    m_ui->m_addFile->setIcon(KIcon("list-add"));
    m_ui->m_addUrl->setIcon(KIcon("list-add"));

    m_ui->m_removePerson->setIcon(KIcon("list-remove"));
    m_ui->m_removeMail->setIcon(KIcon("list-remove"));
    m_ui->m_removeFile->setIcon(KIcon("list-remove"));
    m_ui->m_removeUrl->setIcon(KIcon("list-remove"));

    //set ID to remove buttons
    m_ui->m_removeButtons->setId(m_ui->m_removePerson, 0);
    m_ui->m_removeButtons->setId(m_ui->m_removeMail, 1);
    m_ui->m_removeButtons->setId(m_ui->m_removeFile, 2);
    m_ui->m_removeButtons->setId(m_ui->m_removeUrl, 3);

    // 'dissociate' SLOT
    connect(m_ui->m_removeButtons, SIGNAL( buttonClicked( int ) ), this, SLOT(slot_dissociate( int )));

    // 'associate' SLOT
    connect(m_ui->m_addPerson, SIGNAL(clicked()), this, SLOT(slot_bindPerson()));
    //connect( m_ui->m_addMail, SIGNAL( clicked() ), this, SLOT( slot_bindMail() ) );
    connect(m_ui->m_addFile, SIGNAL(clicked()), this, SLOT(slot_bindFile()));
    connect(m_ui->m_addUrl, SIGNAL(clicked()), this, SLOT(slot_bindUrl()));

    // enable 'dissociate' buttons
    connect( m_ui->m_persons, SIGNAL( clicked( const QModelIndex &) ), this, SLOT( slot_enableDissociatebuttons() ));
    connect( m_ui->m_emails, SIGNAL( clicked( const QModelIndex & ) ), this, SLOT( slot_enableDissociatebuttons() ));
    connect( m_ui->m_files, SIGNAL( clicked( const QModelIndex & ) ), this, SLOT( slot_enableDissociatebuttons() ));
    connect( m_ui->m_urls, SIGNAL( clicked( const QModelIndex & ) ), this, SLOT( slot_enableDissociatebuttons() ));
}

MainWindow::MainWindow(QWidget *parent) :
    KMainWindow(parent), m_ui(new Ui::MainWindow)
{
    setAutoSaveSettings();
    m_ui->setupUi(this);

    //unsignifacant column and header
    const int columnCount = m_ui->m_tasksview->model()->columnCount();
    for (int i = 1; i < columnCount; i++)
        m_ui->m_tasksview->setColumnHidden(i, true);

    m_ui->m_tasksview->setDirOnlyMode(true);

    Controller* C = new Controller(this);
    C->addModel(m_ui->m_persons, Controller::Person);
    C->addModel(m_ui->m_emails, Controller::EMail);
    C->addModel(m_ui->m_files, Controller::File);
    C->addModel(m_ui->m_urls, Controller::Url);

    setupButtons();

    // double click on an item
    connect( m_ui->m_tasksview, SIGNAL(activated ( const KUrl & ) ),
             this, SLOT( slot_taskActivated( const KUrl & ) ));
    connect( m_ui->m_persons, SIGNAL(activated ( const QModelIndex & ) ),
             this, SLOT( slot_personActivated( const QModelIndex & ) ));
    connect( m_ui->m_emails, SIGNAL(activated ( const QModelIndex & ) ),
             this, SLOT( slot_mailActivated( const QModelIndex & ) ));
    connect( m_ui->m_files, SIGNAL(activated ( const QModelIndex & ) ),
             this, SLOT( slot_resourceActivated( const QModelIndex & ) ));
    connect( m_ui->m_urls, SIGNAL(activated ( const QModelIndex & ) ),
             this, SLOT( slot_resourceActivated( const QModelIndex & ) ));

    //when the current task changes
    connect(m_ui->m_tasksview, SIGNAL(currentUriChanged( const KUrl& )),
            C, SLOT( slot_currentChanged( const KUrl& )));
    //when the models need to update (new resource or new relation in the nepomuk db)
    connect(this, SIGNAL( syncWithNepomukDB( const KUrl& ) ),
            C, SLOT( slot_currentChanged( const KUrl& ) ));

    // Custom Context Menu
    connect(m_ui->m_tasksview, SIGNAL(customContextMenuRequested ( const QPoint & )),
            this, SLOT( slot_customContextMenuRequested_taskview( const QPoint & )));
    connect(m_ui->m_persons, SIGNAL(customContextMenuRequested ( const QPoint & )),
            this, SLOT( slot_customContextMenuRequested( const QPoint & )));
    connect(m_ui->m_emails, SIGNAL(customContextMenuRequested ( const QPoint & )),
            this, SLOT( slot_customContextMenuRequested( const QPoint & )));
    connect(m_ui->m_files, SIGNAL(customContextMenuRequested ( const QPoint & )),
            this, SLOT( slot_customContextMenuRequested( const QPoint & )));
    connect(m_ui->m_urls, SIGNAL(customContextMenuRequested ( const QPoint & )),
            this, SLOT( slot_customContextMenuRequested( const QPoint & )));

    //// TOOLBAR
    QAction *tmp;

    tmp = m_ui->m_toolbar->addAction(KIcon("task-new"), i18n("New &Task"), this, SLOT(slot_newTask()));
    tmp->setShortcut(QString("Ctrl+Shift+T"));
    tmp->setToolTip(QString("Create new Nepomuk Task\nCtrl+Shift+T"));

    tmp = m_ui->m_toolbar->addAction(KIcon("contact-new"), i18n("New Contact"), this, SLOT(slot_newPerson()));
    tmp->setShortcut(QKeySequence( QLatin1String("Ctrl+Shift+C")) );
    tmp->setToolTip(i18n("Create a new Nepomuk Contact\nCtrl+Shift+C"));

    // tmp = m_ui->m_toolbar->addAction(KIcon("user-group-new"), i18n("New Organization"), this, SLOT(slot_newOrganization()));
    // tmp->setShortcut(QString("Ctrl+Shift+O"));
    // tmp->setToolTip(QString("Create a new Organization\nCtrl+Shift+C"));

    QToolButton* associateButton = new QToolButton(this);
    associateButton->setIcon(KIcon("nepomuk"));
    associateButton->setText(i18n("Associate"));
    associateButton->setToolButtonStyle(Qt::ToolButtonTextUnderIcon);
    associateButton->setPopupMode(QToolButton::InstantPopup);
    associateButton->setToolTip(i18n("Associate resources to tasks"));

    m_ui->m_toolbar->addWidget(associateButton);
    associateButton->setEnabled(false);
    connect( m_ui->m_tasksview, SIGNAL( clicked( const QModelIndex & ) ), this, SLOT(slot_enableAssociateButtons( const QModelIndex & )) );
    connect( this, SIGNAL(taskSelected( bool )), associateButton, SLOT( setEnabled( bool ) ) );

    QMenu* menu = new QMenu(this);

    tmp = menu->addAction(KIcon("view-pim-contacts"), i18n("Contact"), this, SLOT(slot_bindPerson()));
    tmp->setShortcut(QKeySequence( QLatin1String("Alt+Shift+C")) );
    tmp->setToolTip(i18n("Associate a contact\nAlt+Shift+C"));

    tmp = menu->addAction(KIcon("mail-mark-unread"), i18n("Email"), this, SLOT(slot_newMail()));
    tmp->setShortcut(QKeySequence( QLatin1String("Alt+Shift+M")) );
    tmp->setToolTip(i18n("Associate a mail\nAlt+Shift+M"));
    tmp->setEnabled(false);

    tmp = menu->addAction(KIcon("message-news"), i18n("File"), this, SLOT(slot_bindFile()));
    tmp->setShortcut(QKeySequence( QLatin1String("Alt+Shift+F")) );
    tmp->setToolTip(i18n("Associate a file\nAlt+Shift+F"));

    tmp = menu->addAction(KIcon("text-html"), i18n("Web Page"), this, SLOT(slot_bindUrl()));
    tmp->setShortcut(QKeySequence( QLatin1String("Alt+Shift+W")) );
    tmp->setToolTip(i18n("Associate a Web page\nAlt+Shift+W"));
    associateButton->setMenu(menu);

    //tmp = m_ui->m_toolbar->addAction( i18n("Export tasks"), this, SLOT(slot_exportTasks() ) );

    //tmp = m_ui->m_toolbar->addAction( i18n("Import tasks"), this, SLOT(slot_importTasks() ) );


    //// VIEW CONTROL
    // m_ui->m_toolbar->addSeparator();
    // QAction* expandAction = m_ui->m_toolbar->addAction(KIcon("expand"), i18n("Expand all"), this, SLOT(slot_expandAll()));
    // expandAction->setShortcut(QString("Alt+X"));
    // expandAction->setToolTip(QString("Expand the task tree\nAlt+X"));

    //// ABOUT
    m_ui->m_toolbar->addSeparator();
    m_ui->m_toolbar->addAction(KIcon("help-about"), i18n("About"), this, SLOT(slot_about()));

    connect(Nepomuk::ResourceManager::instance()->mainModel(), SIGNAL( statementsAdded() ),
            this, SLOT(statementsAddedOrRemoved()));
    connect(Nepomuk::ResourceManager::instance()->mainModel(), SIGNAL( statementsRemoved() ),
            this, SLOT(statementsAddedOrRemoved()));

    readSettings();

}

MainWindow::~MainWindow()
{
    delete m_ui;
}

void MainWindow::slot_customContextMenuRequested_taskview(const QPoint& pos)
{
    QModelIndex index = m_ui->m_tasksview->indexAt(pos);

    if (index.isValid()) {
        Nepomuk::Resource task = m_ui->m_tasksview->currentUri();

        QMenu menu;
        QAction* OpenInDolphin = menu.addAction( KIcon( "folder" ), i18n( "Open in Dolphin" ));
        QAction* Annotate = menu.addAction( KIcon( "nepomuk" ), i18n( "Annotate" ));
        menu.addAction( i18n( "Expand" ), this, SLOT( slot_expandAll() ) );
        QMenu* sub = new QMenu(i18n( "Create and associate" ), &menu);
        QAction* newPerson = sub->addAction( KIcon( "view-pim-contacts" ), i18n( "Contact" ));
        QAction* newMail = sub->addAction( KIcon( "mail-mark-unread" ), i18n( "Email" ));
        QAction* newFile = sub->addAction( KIcon( "message-news" ), i18n( "File" ));
        QAction* newUrl = sub->addAction( KIcon( "text-html" ), i18n( "Web Page" ));
        menu.addMenu(sub);
        menu.addSeparator();
        newMail->setEnabled( false );
        QAction* Delete = menu.addAction(KIcon( "edit-delete" ), i18n( "Delete" ));

        QAction* a = menu.exec(m_ui->m_tasksview->viewport()->mapToGlobal(pos));

        if (a == OpenInDolphin) {
            KProcess::startDetached( "dolphin", QStringList(m_ui->m_tasksview->currentUrl().pathOrUrl()));
        } else if (a == Annotate) {
            KProcess::startDetached( "nepomuksimpleannotator", QStringList(task.resourceUri().toString()));
        } else if (a == newPerson) {
            Nepomuk::Resource Person = NewPersonDialog::createNewPerson();
            if (Person.isValid()) {
                Person.addProperty(Nepomuk::Vocabulary::PIMO::isRelated(),
                                   Nepomuk::Resource(m_ui->m_tasksview->currentUri()));
                //emit syncWithNepomukDB( m_ui->m_tasksview->currentUri() );
            }
        } else if (a == newUrl) {
            Nepomuk::Resource Website = NewUrlDialog();
            if (Website.isValid()) {
                Website.addProperty(Nepomuk::Vocabulary::PIMO::isRelated(),
                                    Nepomuk::Resource(m_ui->m_tasksview->currentUri()));
                //emit syncWithNepomukDB( m_ui->m_tasksview->currentUri() );
            }
        } else if (a == newFile) {
            slot_bindFile();
        } else if (a == Delete) {
            // this will remove the task and the links to and from it
            task.remove();
            kDebug() << "removing" << m_ui->m_tasksview->currentUrl().url();
            org::kde::KDirNotify::emitFilesRemoved(QStringList(m_ui->m_tasksview->currentUrl().url()));
        }
    }
}

void MainWindow::slot_customContextMenuRequested(const QPoint& pos) {
    QListView* listview = static_cast<QListView*> (sender());
    QModelIndex index = listview->indexAt(pos);
    const int selectedRowsCount =
        listview->selectionModel()->selectedRows().size();

    //if ( index.isValid() ) {
    if (selectedRowsCount == 1) { // single row selection handling
        Nepomuk::Resource res(index.data(
                                  Nepomuk::ResourceModel::ResourceUriRole).toUrl());
        // kDebug() << "Current Resource" << res.resourceUri();
        // kDebug() << "Current Url" << m_ui->m_tasksview->currentUrl().pathOrUrl();

        QMenu menu;

        QAction* Open = menu.addAction(i18n("Open with the default application"));
        QAction* OpenInKonqueror = 0;
        QAction* OpenInDolphin = 0;
        QAction* OpenInKMail = 0;
        QAction* SendMail = 0;

        if (listview == m_ui->m_urls)
            OpenInKonqueror = menu.addAction(KIcon("konqueror"), i18n("Open in Konqueror"));
        else if (listview == m_ui->m_files)
            OpenInDolphin = menu.addAction(KIcon("folder"), i18n("Open containg folder in Dolphin"));
        else if (listview == m_ui->m_emails)
            OpenInKMail = menu.addAction(KIcon("kmail"), i18n("Open in KMail"));

        menu.addSeparator();
        if (listview == m_ui->m_persons) {
            SendMail = menu.addAction(KIcon("mail-message-new"), i18n("Send an e-mail")); // if is of type 'person'
            if (res.hasProperty(Nepomuk::Vocabulary::NCO::EmailAddress()) == false) // ... but does not have an email
                SendMail->setEnabled(false); // disable the email feature
            Open->setVisible( false );
        }

	if ( OpenInKMail && !res.hasProperty(QUrl::fromEncoded(nmoMessageId) ) ) {
	  OpenInKMail->setEnabled(false); //if is a mail but not created from kmail, disabling opening with Kmail
	}

        QAction* Annotate = menu.addAction(KIcon("nepomuk"), i18n("Annotate"));
        menu.addSeparator();
        QAction* RemoveFrom = menu.addAction(KIcon("list-remove"), i18n("Dissociate from '%1'", Nepomuk::Resource(m_ui->m_tasksview->currentUri()).genericLabel()));
        QAction* Delete = menu.addAction(KIcon("edit-delete"), i18n("Delete"));

        if (listview == m_ui->m_files)
            Delete->setVisible(false);

        QAction* a = menu.exec(listview->viewport()->mapToGlobal(pos));

        if (a == 0)
            return;
        if (a == Open) {
            bool isLocalFile = !res.hasType(Nepomuk::Vocabulary::NFO::Website());
            new KRun(res.resourceUri(), this, 0, isLocalFile);
        }
        else if (a == OpenInDolphin) {
            KProcess::startDetached( "dolphin", QStringList( KUrl( res.resourceUri() ).upUrl().url()) );
        }
        else if (a == OpenInKonqueror) {
            KProcess::startDetached( "konqueror", QStringList(res.resourceUri().toString() ));
        }
        else if (a == OpenInKMail) {
	  KProcess::startDetached( "kmail", QStringList() << "--viewid" <<
				   res.property(QUrl::fromEncoded(nmoMessageId) ).toString() );
        }
        else if (a == Annotate) {
            KProcess::startDetached( "nepomuksimpleannotator", QStringList( res.resourceUri().toString() ));
        }
        else if (a == SendMail) {
            KProcess::startDetached( "kmail", QStringList( res.property( Nepomuk::Vocabulary::NCO::EmailAddress()).toString() ));
        }
        else if (a == Delete) {
            if (KMessageBox::questionYesNo(this, i18n("Do you confirm the definitive removal of %1 ?", res.genericLabel())) == KMessageBox::Yes )
                res.remove();
            //emit syncWithNepomukDB( m_ui->m_tasksview->currentUri() );
        }
        else if (a == RemoveFrom) {
            res.removeProperty(Nepomuk::Vocabulary::PIMO::isRelated(),
                               Nepomuk::Resource(m_ui->m_tasksview->currentUri()));
            if (listview == m_ui->m_files) {
                kDebug() << "removing file" << res;
                res.remove();
            }
            //emit syncWithNepomukDB( m_ui->m_tasksview->currentUri() );
        }
    }

    else if (selectedRowsCount > 1) { // multiple row selection
        QMenu menu;
        QAction* SendMail = 0;
        QAction* OpenInKonqueror = 0;
        if (sender() == m_ui->m_persons) {
            SendMail = menu.addAction(KIcon("mail-message-new"), i18n("Send an e-mail"));
        }
        if (sender() == m_ui->m_urls) {
            OpenInKonqueror = menu.addAction(KIcon("konqueror"), i18n("Open in Konqueror"));
        }
        menu.addSeparator();
        QAction* Delete = menu.addAction(KIcon("edit-delete"), i18n("Delete"));
        QAction* RemoveFrom = menu.addAction(KIcon("list-remove"), i18n("Dissociate from '%1'", Nepomuk::Resource(m_ui->m_tasksview->currentUri()).genericLabel()));

        if (listview == m_ui->m_files)
            Delete->setVisible(false);

        QList<Nepomuk::Resource> list = getListSelectedResources(static_cast<QListView*> (sender()));

        QAction* a = menu.exec(static_cast<QAbstractItemView*> (sender())->viewport()->mapToGlobal(pos));
        if (a == 0 )
            return ;
        if ( a == SendMail ) {
            QStringList args;
            foreach (Nepomuk::Resource res, list) { // FIXME: warn for persons which do not have an email address
                args << res.property( Nepomuk::Vocabulary::NCO::EmailAddress() ).toString();
            }
            KProcess::startDetached("kmail", args);
        }
        else if ( a == OpenInKonqueror ) {
            QStringList args;
            foreach (Nepomuk::Resource res, list) {
                args << res.resourceUri().toString();
            }
            KProcess::startDetached("konqueror", args);
        }
        else if (a == Delete) {
            foreach( Nepomuk::Resource res, list )
                res.remove();
            //emit syncWithNepomukDB( m_ui->m_tasksview->currentUri() );
        }
        else if ( a == RemoveFrom ) {
            foreach( Nepomuk::Resource res, list )
                res.removeProperty(Nepomuk::Vocabulary::PIMO::isRelated(), Nepomuk::Resource( m_ui->m_tasksview->currentUri()) );
            //emit syncWithNepomukDB( m_ui->m_tasksview->currentUri() );
        }
    }
}

inline void MainWindow::slot_exportTasks()
{
    const KUrl archivePath = KFileDialog::getSaveUrl( KUrl("kfiledialog:///tasktopexport"), "", this, i18n("Task archive location and name") );
    if ( !archivePath.isEmpty() ) {
        taskExport( archivePath );
    }
}

inline void MainWindow::slot_importTasks()
{
    const KUrl archivePath = KFileDialog::getOpenFileName( KUrl("kfiledialog:///tasktopimport"), "*.nepomuk.tar.bz2", this, i18n("Task archive to be imported") );
    if ( !archivePath.isEmpty() ) {
        const KUrl extractLocation =
            KFileDialog::getExistingDirectoryUrl( KUrl("kfiledialog:///tasktopimport2"),
                                                  this, i18n("Location of the imported documents") );
        if ( !extractLocation.isEmpty() ) {
            taskImport( archivePath, extractLocation );
        }
    }
}

inline void MainWindow::slot_newTask() {
    Nepomuk::Resource Task = NewTaskDialog::createNewTask(this);
    if (Task.isValid()) {
        KUrl url = Controller::FromUriToNepomukTreeUrl(Task.resourceUri());
        url = url.directory();
        url.setProtocol("nepomuktree");
        kDebug() << "url to refresh" << url;
        org::kde::KDirNotify::emitFilesAdded(url.url());
    }
}

inline void MainWindow::slot_newPerson() {
    NewPersonDialog::createNewPerson(this);
}

inline void MainWindow::slot_newOrganization()
{
    NewOrganizationDialog::CreateNewOrganization(this);
}

inline void MainWindow::slot_about()
{
    //KMessageBox::about(this, i18n(), i18n("About Tasktop"));
    KAboutApplicationDialog( KGlobal::mainComponent().aboutData(), this ).exec();
}

inline void MainWindow::slot_expandAll()
{
    static bool collapsed = true;
    collapsed ? m_ui->m_tasksview->expandAll() : m_ui->m_tasksview->collapseAll();
    //m_ui->m_tasksview->collapseAll( collapsed );
    static_cast<QAction*>( sender() )->setText( collapsed ? "Expand All" : "Collapse All" );
    collapsed = !collapsed;
}

inline void MainWindow::slot_newMail() {
    KProcess::startDetached("kontact --module kmailplugin");
}

inline void MainWindow::slot_newUrl() {
    NewUrlDialog();
}

void MainWindow::slot_bindPerson() {
    simpleResourceBinder(Nepomuk::Vocabulary::NCO::Contact());
    //emit syncWithNepomukDB( m_ui->m_tasksview->currentUri() );
}

void MainWindow::slot_bindUrl() {
    simpleResourceBinder(Nepomuk::Vocabulary::NFO::Website());
    //emit syncWithNepomukDB( m_ui->m_tasksview->currentUri() );
}

void MainWindow::slot_bindFile() {
    KUrl::List Urls = KFileDialog::getOpenUrls();

    if (!Urls.isEmpty()) {
        Nepomuk::Resource Task = m_ui->m_tasksview->currentUri();
        foreach(KUrl url, Urls) {
            kDebug() << url;
            Nepomuk::Resource File( url, Nepomuk::Vocabulary::NFO::FileDataObject() );
            File.setLabel( url.fileName() );
            File.addProperty( Nepomuk::Vocabulary::PIMO::isRelated(), Task );
        }
        emit syncWithNepomukDB(m_ui->m_tasksview->currentUri());
    }
}

//void MainWindow::slot_hideButtons(QAbstractButton* button) {
//	button->setIcon(button->isChecked() ? KIcon("arrow-up-double") : KIcon(
//			"arrow-down-double"));
//}

void MainWindow::slot_enableAssociateButtons(const QModelIndex&)
{
    emit this->taskSelected( true );
    disconnect( SIGNAL(taskSelected( bool )), this, SLOT(slot_enableAssociateButtons( const QModelIndex& )) );

    m_ui->m_addPerson->setEnabled(true);
    //m_ui->m_addMail->setEnabled( true );
    m_ui->m_addFile->setEnabled(true);
    m_ui->m_addUrl->setEnabled(true);
}

void MainWindow::slot_enableDissociatebuttons() //FIXME: not infaillible
{
    QObject* Sender = sender();
    if (Sender == m_ui->m_persons)
        m_ui->m_removePerson->setEnabled(true);
    else if (Sender == m_ui->m_emails)
        m_ui->m_removeMail->setEnabled(true);
    else if (Sender == m_ui->m_files)
        m_ui->m_removeFile->setEnabled(true);
    else if (Sender == m_ui->m_urls)
        m_ui->m_removeUrl->setEnabled(true);
}

void MainWindow::slot_dissociate(int id)
{
    QListView* listview = 0;

    switch (id) {
    case 0:
        listview = m_ui->m_persons;
        break;
    case 1:
        listview = m_ui->m_emails;
        break;
    case 2:
        listview = m_ui->m_files;
        break;
    case 3:
        listview = m_ui->m_urls;
        break;
    default:
        break;
    }

    if (!listview->selectionModel()->selectedRows().isEmpty()) {
        QListIterator<QModelIndex>
            i(listview->selectionModel()->selectedRows());

        Nepomuk::Resource task = m_ui->m_tasksview->currentUri();

        while (i.hasNext()) { //FIXME:soprano ask to have an Uri, but if so, it does not work
            Nepomuk::Resource res(i.next().data(
                                      Nepomuk::ResourceModel::ResourceUriRole).toUrl());
            //Nepomuk::Resource( i.next().data().toUrl()).removeProperty( Nepomuk::Vocabulary::PIMO::isRelated(), task);//m_ui->m_tasksview->currentUri() );

            kDebug() << "removing " << res << " from " << task;
            res.removeProperty(Nepomuk::Vocabulary::PIMO::isRelated(), task);

            if (listview == m_ui->m_files) //if it's a file...
                if (!res.hasProperty(Nepomuk::Vocabulary::PIMO::isRelated())) // ... and it not related anymore to anything
                    res.remove(); // remove it from DB
        }
    }
}

Nepomuk::Resource MainWindow::NewUrlDialog() {
    KUrl url = QInputDialog::getText(this, i18n("New Web Page"), i18n("Enter the URL address of the Web site"));
    if (!url.isEmpty() && url.isValid()) {
        url = KUrl("//" + url.url()); // FIXME: proper way must exist
        kDebug() << url;
        if (url.protocol() != "http")
            url.setProtocol("http");
        Nepomuk::Resource website(url, Nepomuk::Vocabulary::NFO::Website());
        website.setLabel(url.host());
        return website;
    }
    return Nepomuk::Resource();
}

void MainWindow::statementsAddedOrRemoved()
{
    emit syncWithNepomukDB(m_ui->m_tasksview->currentUri());
}

void MainWindow::simpleResourceBinder(const Nepomuk::Types::Class& type)
{
    KDialog dlg;
    SimpleResourceListView* srlv = new SimpleResourceListView( &dlg );
    srlv->setResources(Controller::getResourcesOfTypeWithExclusion( type, m_ui->m_tasksview->currentUri()) );

    dlg.setCaption(i18n("Associate"));
    dlg.setButtons(KDialog::Ok | KDialog::Cancel);
    dlg.setMainWidget(srlv);
    dlg.setMinimumSize(430, 300);

    connect( srlv, SIGNAL( doubleClicked( const QModelIndex& ) ),
             &dlg, SLOT( accept() ) );

    if (dlg.exec() == KDialog::Accepted) {
        QList<Nepomuk::Resource> Resources = srlv->getSelectedResources(); //TODO: remove this intermediate variable
        Nepomuk::Resource Task = m_ui->m_tasksview->currentUri();
        QMutableListIterator<Nepomuk::Resource> i(Resources);//srlv->getSelectedResources() );

        while (i.hasNext()) {
            i.next().addProperty(Nepomuk::Vocabulary::PIMO::isRelated(), Task);
        }
    }
}

void MainWindow::slot_taskActivated( const KUrl & url )
{
    kDebug() << "TASK URL" << url;
    Nepomuk::Resource Task( m_ui->m_tasksview->currentUri() );
    NewTaskDialog::edit( Task );

    org::kde::KDirNotify::emitFilesChanged( QStringList( Controller::FromUriToNepomukTreeUrl( Task.resourceUri()).url() ) );
}

void MainWindow::slot_personActivated( const QModelIndex& index )
{
    Nepomuk::Resource Person( index.data( Nepomuk::ResourceModel::ResourceUriRole ).toUrl());
    NewPersonDialog::edit( Person );
}

void MainWindow::slot_mailActivated( const QModelIndex& index )
{
  Nepomuk::Resource mail = index.data( Nepomuk::ResourceModel::ResourceUriRole ).toUrl();
  if ( mail.hasProperty( QUrl::fromEncoded(nmoMessageId) ) ) {
    KProcess::startDetached("kmail", QStringList() << "--viewid" << mail.
			    property( QUrl::fromEncoded(nmoMessageId) ).toString() );
  }
  else {
    KMessageBox::error(this, i18n("this mail has not been added to nepomuk using kmail\n\
Therefore you cannot open it using kmail") );
  }
}

void MainWindow::slot_resourceActivated( const QModelIndex& index ) //only for resource that have a relevant URI (webpage url, file url)
{
    Nepomuk::Resource res ( index.data( Nepomuk::ResourceModel::ResourceUriRole ).toUrl() );
    bool isLocalFile = !res.hasType(Nepomuk::Vocabulary::NFO::Website());
    new KRun( res.resourceUri(), this, 0, isLocalFile );
}

void MainWindow::changeEvent(QEvent *e) {
    QMainWindow::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        m_ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void MainWindow::readSettings() {
    QSettings settings("nepomuk", "tasktop");
    restoreState(settings.value("MainWindow").toByteArray());
}

void MainWindow::writeSettings() {
    QSettings settings("nepomuk", "tasktop");
    settings.setValue("MainWindow",saveState());
}


void MainWindow::closeEvent(QCloseEvent *event) {
    //if (maybeSave()) {
    writeSettings();
    event->accept();
    //     } else {
    //         event->ignore();
    //     }
}

#include "mainwindow.moc"
