/*
   Copyright (C) 2009 by Sebastian Trueg <trueg at kde.org>
   Copyright (C) 2009 by Charles Wautier <charles.wautier@gmail.com>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include "newpersondialog.h"
#include "ui_newpersondialog.h"
#include "tmo.h"
#include "nco.h"
#include "tasktreedialog.h"

#include "nepomukcontext.h"
#include <Nepomuk/ResourceManager>
#include <Nepomuk/Resource>
#include <Nepomuk/Variant>
#include "pimo.h"

#include <KDialog>

#include <KDebug>

class NewPersonDialog::Private : public Ui::NewPersonDialog
{
public:
    Nepomuk::Resource otherTask;
    Nepomuk::Resource currentContext;
    Nepomuk::ContextServiceInterface* context;
};

NewPersonDialog::NewPersonDialog( QWidget* parent )
    : KDialog( parent ), d( new Private )
{
    Nepomuk::ResourceManager::instance()->init();

    // Dialog conf
    setButtons( Ok|Cancel );
    enableButtonOk( false );
    setDefaultButton( KDialog::Ok );
    setPlainCaption(i18n ("New Contact"));

    // Ui conf
    d->setupUi( mainWidget() );
    d->m_firstName->setFocus();
    d->m_iconButton->setIcon( "view-pim-contacts" );
    connect( d->m_lastName, SIGNAL( textChanged( const QString& ) ),
             this, SLOT( slotLabelChanged( const QString& ) ) );

}


NewPersonDialog::~NewPersonDialog()
{
    delete d;
}

void NewPersonDialog::slotLabelChanged( const QString& text )
{
    enableButtonOk( !text.isEmpty() );
}

void NewPersonDialog::edit( Nepomuk::Resource& Person, QWidget* parent )
{
  Q_ASSERT( Person.hasType( Nepomuk::Vocabulary::NCO::Contact() ) == false );
  NewPersonDialog dlg( parent );

  dlg.setCaption( i18n( "Edit contact" ) );
  readProperties( dlg, Person );
  if ( dlg.exec() == Accepted ) {
    writeProperties( dlg, Person );
  }
}

Nepomuk::Resource NewPersonDialog::createNewPerson( QWidget* parent )
{
    NewPersonDialog dlg( parent );

    if ( dlg.exec() == Accepted ) {
      Nepomuk::Resource Person( dlg.d->m_firstName->text() + dlg.d->m_lastName->text(), Nepomuk::Vocabulary::NCO::Contact() );
      writeProperties( dlg, Person );
      return Person;
    }
    else {
      return Nepomuk::Resource();
    }
}

void NewPersonDialog::readProperties( NewPersonDialog& dlg, const Nepomuk::Resource& Person )
{
  if ( Person.symbols().size() != 0 )
    dlg.d->m_iconButton->setIcon ( Person.symbols().first() );

  dlg.d->m_firstName->setText( Person.property(Nepomuk::Vocabulary::NCO::nameGiven()).toString() );
  dlg.d->m_lastName->setText( Person.property(Nepomuk::Vocabulary::NCO::nameFamily()).toString() );

  dlg.d->m_description->setText (Person.description() );

  if ( Person.hasProperty( Nepomuk::Vocabulary::NCO::emailAddress()) )
    dlg.d->m_email->setText( Person.property(Nepomuk::Vocabulary::NCO::emailAddress() ).toString() );

  if ( Person.hasProperty( Nepomuk::Vocabulary::NCO::websiteUrl()) )
    dlg.d->m_website->setText( Person.property(Nepomuk::Vocabulary::NCO::websiteUrl() ).toString() );

  if ( Person.hasProperty( Nepomuk::Vocabulary::NCO::PhoneNumber()) )
    dlg.d->m_phone->setText( Person.property(Nepomuk::Vocabulary::NCO::PhoneNumber() ).toString() );

  if ( Person.hasProperty( Nepomuk::Vocabulary::NCO::birthDate()) ) {
    dlg.d->m_birthDateBox->setChecked( true );
    dlg.d->m_birthDate->setEnabled( true );
    dlg.d->m_birthDate->setDate( Person.property( Nepomuk::Vocabulary::NCO::birthDate() ).toDate() );
  }
}

void NewPersonDialog::writeProperties(const NewPersonDialog& dlg, Nepomuk::Resource& Person )
{
  Person.setLabel( dlg.d->m_firstName->text() + " " + dlg.d->m_lastName->text() );
  Person.setProperty( Nepomuk::Vocabulary::NCO::nameFamily(), dlg.d->m_lastName->text() );
  Person.setProperty( Nepomuk::Vocabulary::NCO::nameGiven(), dlg.d->m_firstName->text() );

  if ( !dlg.d->m_description->toPlainText().isEmpty() ) {
    Person.setDescription( dlg.d->m_description->toPlainText() );
  }
  //workaround for simpleresource model icon changing bug
  if ( Person.symbols().contains( dlg.d->m_iconButton->icon()) == false ) {
    Person.setSymbols( QStringList(dlg.d->m_iconButton->icon()) );
  } 

  if ( !dlg.d->m_email->text().isEmpty() ) //TODO: Check mail adress valabitily
    Person.setProperty( Nepomuk::Vocabulary::NCO::emailAddress(), dlg.d->m_email->text() );

  if ( !dlg.d->m_website->text().isEmpty() ) {
    Person.setProperty( Nepomuk::Vocabulary::NCO::websiteUrl(), dlg.d->m_website->text()  );
  }
  if ( !dlg.d->m_phone->text().isEmpty() ) {
    Person.setProperty( Nepomuk::Vocabulary::NCO::PhoneNumber(), dlg.d->m_phone->text()  );
  }
  if ( dlg.d->m_birthDateBox->isChecked() ) {
    Person.setProperty( Nepomuk::Vocabulary::NCO::birthDate(), dlg.d->m_birthDate->date() );
  }
}

#include "newpersondialog.moc"
