/* This file is part of the Nepomuk Project
   Copyright (c) 2009 Sebastian Trueg <trueg@kde.org>
   Copyright (c) 2009 Charles Wautier <charles.wautier@gmail.com>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#include "nepomukcontextchooser.h"
#include "tmo.h"
#include "newtaskdialog.h"
#include "tasktreedialog.h"
#include "kresourcetreeview.h"
#include "kprocess.h"
#include <kdirnotify.h>

#include "nepomukcontext.h"
#include <QPainter>
#include <QtGui/QToolButton>
#include <QtGui/QGraphicsLinearLayout>
#include <QHeaderView>
#include <plasma/widgets/treeview.h>
#include <KLocale>
#include <KMessageBox>

#include <Plasma/ToolButton>
#include <Plasma/Label>
#include <Plasma/FlashingLabel>
#include <Plasma/ToolTipContent>
#include <Plasma/ToolTipManager>
#include <Plasma/Frame>

#include <Nepomuk/Resource>
#include <Nepomuk/ResourceManager>
#include <KPushButton>

#include <Soprano/Model>
#include <Soprano/StatementIterator>
#include <Soprano/NodeIterator>

#include <QColor>
#include <QLabel>
#include <QDirModel>
#include <QTreeView>
#include <QVBoxLayout>
#include <Qt>

#define MINBUTTONSIZE 21
#define MARGINSIZE 2

// FIXME : An approximative solution to refresh the treeviews
static KUrl uriToName( const KUrl& uri )
{
  QByteArray a(uri.url().toAscii());
  KUrl name = QString( a.toPercentEncoding("", "~", '~'));
  kDebug() << name;
  return name;
}
static KUrl FromUriToNepomukTreeUrl( const KUrl& taskuri )
{
  Soprano::Model* model = Nepomuk::ResourceManager::instance()->mainModel();
  KUrl url;
  if ( model->containsAnyStatement( taskuri, Nepomuk::Vocabulary::TMO::superTask(), Soprano::Node::createEmptyNode() ) ) {

    Soprano::StatementIterator statIt = model->listStatements( taskuri, Nepomuk::Vocabulary::TMO::superTask(), Soprano::Node::createEmptyNode() );
    statIt.next();

    KUrl MotherTaskUri = statIt.iterateObjects().current().toString();
    // if ( statIt.next() )
    //   kDebug() << "FATAL ERROR" << "tasks shall always have only one mother task!";
    KUrl grandMother = FromUriToNepomukTreeUrl( MotherTaskUri );
    kDebug() << "recursion returned" <<  grandMother ;

    url = grandMother;
    //kDebug() << "uriToName" << uriToName( url.url() );

    url.addPath( uriToName( taskuri ).url() );
    url.setProtocol( "nepomuktree" );
    kDebug() << "final url" << url.url() << "current uri" << taskuri.url();
    return url;
  }
  KUrl tmp;
  tmp.setProtocol("nepomuktree:");
  tmp.adjustPath( KUrl::AddTrailingSlash );
  tmp.addPath( uriToName( taskuri ).url() );
  return tmp;
}

NepomukContextChooser::NepomukContextChooser(QObject *parent, const QVariantList &args)
    : Plasma::PopupApplet(parent, args), m_widget(0)
{
}

NepomukContextChooser::~NepomukContextChooser()
{
}

void NepomukContextChooser::paintInterface( QPainter *p,
                                            const QStyleOptionGraphicsItem *option, const QRect &contentsRect )
{
    Applet::paintInterface( p, option, contentsRect );
}

void NepomukContextChooser::init()
{
    m_frame = new Plasma::Frame(this);
    m_frame->setFrameShadow( Plasma::Frame::Sunken );

    m_label = new Plasma::Label( m_frame );
    m_label->nativeWidget()->setWordWrap(false);
    m_label->nativeWidget()->setScaledContents(true);
    m_label->nativeWidget()->setText ( QString("  ...  "));
    Plasma::ToolTipContent staskToolTip(i18n("Active Task"),i18n("Desktop Wide Task Setter"),KIcon("edit-node"));
    Plasma::ToolTipManager::self()->setContent(m_label, staskToolTip);
    //connect( m_label->nativeWidget(), SIGNAL( linkActivated( const QString & ) ),
    //         this, SLOT( slotChooseContext( const QString & ) ) );

    m_newTaskButton = new Plasma::ToolButton( m_frame );
    m_newTaskButton->nativeWidget()->setIcon( KIcon( "task-new" ) );
    Plasma::ToolTipContent ntaskToolTip(i18n("New Task"),i18n("Task Creation"),m_newTaskButton->icon());
    Plasma::ToolTipManager::self()->setContent(m_newTaskButton, ntaskToolTip);
    connect( m_newTaskButton->nativeWidget(), SIGNAL( clicked() ),
             this, SLOT( slotNewTask() ) );

    m_taskTopButton = new Plasma::ToolButton( m_frame );
    m_taskTopButton->nativeWidget()->setIcon( KIcon( "view-calendar-list" ) );
    Plasma::ToolTipContent tTopToolTip(i18n("Tasktop"),i18n("Cross-application Task Manager"),m_taskTopButton->icon());
    Plasma::ToolTipManager::self()->setContent(m_taskTopButton, tTopToolTip);
    connect( m_taskTopButton->nativeWidget(), SIGNAL( clicked() ),
             this, SLOT( slotTaskTop() ) );
   
    //QGraphicsLinearLayout* layy = new QGraphicsLinearLayout(Qt::Horizontal);
	//layy->setContentsMargins(0,0,0,0);
	//layy->setSpacing(0);
	//layy->addItem(m_taskTopButton);
	//layy->addItem(m_newTaskButton);
	// layout children
    QGraphicsLinearLayout* lay = new QGraphicsLinearLayout( Qt::Horizontal,m_frame );
    lay->setContentsMargins(0,0,8,0);
	lay->setSpacing(0);
    //lay->setSpacing(0);
    lay->addItem( m_taskTopButton );
    lay->addItem( m_newTaskButton );
    lay->addItem( m_label );
	lay->setItemSpacing(1,8);

	// layout frame
    QGraphicsLinearLayout* layout = new QGraphicsLinearLayout( this );
    layout->setContentsMargins(MARGINSIZE ,MARGINSIZE ,MARGINSIZE ,MARGINSIZE);
    layout->setSpacing(0);
    layout->addItem( m_frame );

    // sizes ...
    //m_frame->setMinimumSize(7*MINBUTTONSIZE,0);
    m_frame->setMaximumSize(160000,160000);
    m_frame->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);

    m_label->setMinimumSize(5*MINBUTTONSIZE,0);
    m_label->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);

    //m_newTaskButton->setMinimumSize(MINBUTTONSIZE,MINBUTTONSIZE);
    m_newTaskButton->setMaximumSize(2*MINBUTTONSIZE,16000);
    m_newTaskButton->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Expanding);

    //m_taskTopButton->setMinimumSize(MINBUTTONSIZE,MINBUTTONSIZE);
    m_taskTopButton->setMaximumSize(2*MINBUTTONSIZE,16000);
    m_taskTopButton->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Expanding);
    
	// setup Nepomuk context connections
    Nepomuk::ResourceManager::instance()->init();
    m_context = new Nepomuk::ContextServiceInterface();
    connect( m_context, SIGNAL( currentContextChanged(Nepomuk::Resource) ),
             this, SLOT( slotContextChanged( Nepomuk::Resource ) ) );
}

QWidget* NepomukContextChooser::widget()
{
	if (!m_widget) {
    	m_widget = new QWidget();
		QVBoxLayout* poplayout = new QVBoxLayout( m_widget );
		poplayout->setMargin(0);
		poplayout->setSpacing(0);

		KResourceTreeView* m_treeView = new KResourceTreeView(m_widget);
		m_treeView->setDirOnlyMode(true);
		m_treeView->setSelectionMode(QAbstractItemView::SingleSelection);

    	poplayout->addWidget( m_treeView );

		connect( m_treeView, SIGNAL( currentUriChanged( const KUrl& ) ),
             	this, SLOT( slotSelectContext( const KUrl& ) ) );
	}
	return m_widget;
}

void NepomukContextChooser::slotContextChanged( const Nepomuk::Resource& res )
{
    m_label->setText(res.genericLabel());
}

void NepomukContextChooser::slotChooseContext( const QString & )
{
    showPopup(0);
}

void NepomukContextChooser::slotNewTask()
{
	if (!Nepomuk::ResourceManager::instance()->init()) {
    	Nepomuk::Resource res = NewTaskDialog::createNewTask();
    	// We send a notification so other task tree views can update
    	if ( res.isValid() ) {
    	    KUrl url = FromUriToNepomukTreeUrl( res.resourceUri() );
    	    url = url.directory();
    	    url.setProtocol( "nepomuktree" );
    	        org::kde::KDirNotify::emitFilesAdded( url.url() );
    	}
    	selectContext( res );
	} else {
        KMessageBox::error( 0, i18n("The connection to the Nepomuk server could not be established. The application will not be able to run properly. Please check from the KDE System Settings that the Nepomuk server has been started.") );
        kError() << "Unable to initialize KMetaData";
	}
}

void NepomukContextChooser::slotTaskTop()
{
	KProcess::startDetached( "tasktop" , QStringList() );
}

void NepomukContextChooser::slotSelectContext( const KUrl & url )
{
    if ( !url.isEmpty() ) {
        selectContext(Nepomuk::Resource( url ));
        hidePopup();
    }
}

void NepomukContextChooser::selectContext (const Nepomuk::Resource& res )
{
    if ( res.isValid() )
    {
        m_context->setCurrentContext(res);
    }
}

#include "nepomukcontextchooser.moc"
