/* This file is part of the Nepomuk Project
   Copyright (c) 2009 Sebastian Trueg <trueg@kde.org>
   Copyright (c) 2009 Charles Wautier <charles.wautier@gmail.com>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#ifndef NEPOMUKCONTEXTCHOOSER_HEADER
#define NEPOMUKCONTEXTCHOOSER_HEADER

#include <Nepomuk/Resource>
#include <Plasma/Applet>
#include <plasma/widgets/pushbutton.h>
#include <plasma/popupapplet.h>

namespace Plasma {
    class ToolButton;
    class Label;
	class Frame;
	class ComboBox;
}

namespace Nepomuk {
    class ContextServiceInterface;
}

class KResourceTreeView;

class NepomukContextChooser : public Plasma::PopupApplet
{
    Q_OBJECT

public:
    NepomukContextChooser( QObject *parent, const QVariantList &args );
    ~NepomukContextChooser();

    void paintInterface( QPainter *painter,
                         const QStyleOptionGraphicsItem *option,
                         const QRect& contentsRect );
	void init();
	virtual QWidget* widget ();

private Q_SLOTS:

    void slotContextChanged( const Nepomuk::Resource& res );
	void slotChooseContext( const QString & );
    void slotNewTask();
	void slotTaskTop();
	void slotSelectContext( const KUrl & );

private:

    Plasma::ToolButton* m_newTaskButton;
    Plasma::ToolButton* m_taskTopButton;
    Plasma::Label* m_label;
	Plasma::Frame* m_frame;

	QWidget* m_widget;

	Nepomuk::ContextServiceInterface* m_context;

    void selectContext(const Nepomuk::Resource& res);
};

// This is the command that links your applet to the .desktop file
K_EXPORT_PLASMA_APPLET(nepomukcontextchooser, NepomukContextChooser)

#endif
