/* This file is part of the Nepomuk Project
   Copyright (c) 2008 Sebastian Trueg <trueg@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#include "nepomukusercontextservice.h"
#include "usercontextserviceadaptor.h"

#include <KUrl>
#include <KLocale>
#include <KPluginFactory>
#include <KDebug>

#include <Soprano/Model>
#include <Soprano/StatementIterator>
#include <Soprano/Statement>
#include <Soprano/Node>
#include <Soprano/NodeIterator>
#include <Soprano/QueryResultIterator>

#include <Nepomuk/Resource>
#include <Nepomuk/ResourceManager>
#include <Nepomuk/Variant>


/**
 * this marco creates a service factory which can then be found by the Qt/KDE
 * plugin system in the Nepomuk server.
 */
NEPOMUK_EXPORT_SERVICE( Nepomuk::UserContextService, "nepomukusercontextservice")


Nepomuk::UserContextService::UserContextService( QObject* parent, const QList<QVariant>& )
    : Service( parent )
{
    // only so ResourceManager won't open yet another connection to the nepomuk server
    ResourceManager::instance()->setOverrideMainModel( mainModel() );

    ( void )new UserContextServiceAdaptor( this );
}


Nepomuk::UserContextService::~UserContextService()
{
}


QString Nepomuk::UserContextService::currentUserContext()
{
    return QString::fromAscii( m_currentContext.toEncoded() );
}


void Nepomuk::UserContextService::setCurrentUserContext( const QString& uri )
{
    QUrl newContext = QUrl::fromEncoded( uri.toAscii() );
    if ( newContext != m_currentContext ) {
        m_currentContext = newContext;
        emit currentUserContextChanged( uri );
    }
}

#include "nepomukusercontextservice.moc"
